@@ -17,6 +17,8 @@
 
 #if YAZ_HAVE_ICU
 
+#include <libxml/parser.h>
+
 #include <unicode/ucnv.h>
 #include <unicode/ustring.h>
 #include <unicode/ucol.h>
@@ -200,7 +202,7 @@ static void print_icu_transliterators(const struct config_t *p_config)
                 "   Set Complement  [[:Greek:] - [:letter:]]\n"
                 "   Complement      [^[:Greek:] [:letter:]]\n"
                 "\n"
-             "see: http://icu.sourceforge.net/userguide/unicodeSet.html\n"
+             "see: http://userguide.icu-project.org/strings/unicodeset\n"
                 "\n"
                 "Examples:\n"
                 "   [:Punctuation:] Any-Remove\n"
@@ -479,9 +481,11 @@ static void process_text_file(struct config_t *p_config)
             else
             {
                 size_t start, len;
+                const char *org_string = 0;
                 const char *sortkey = icu_chain_token_sortkey(p_config->chain);
 
-                icu_chain_get_org_info(p_config->chain, &start, &len);
+                icu_chain_get_org_info2(p_config->chain, &start, &len,
+                                        &org_string);
                 wrbuf_rewind(sw);
                 wrbuf_puts_escaped(sw, sortkey);
                 token_count++;
@@ -525,6 +529,12 @@ static void process_text_file(struct config_t *p_config)
                     {
                         fprintf(p_config->outfile, " %ld+%ld",
                                 (long) start, (long) len);
+                        fputc(' ', p_config->outfile);
+                        fwrite(org_string, 1, start, p_config->outfile);
+                        fputc('*', p_config->outfile);
+                        fwrite(org_string + start, 1, len, p_config->outfile);
+                        fputc('*', p_config->outfile);
+                        fputs(org_string + start + len, p_config->outfile);
                     }
                     fprintf(p_config->outfile, "\n");
                 }
@@ -556,10 +566,10 @@ int main(int argc, char **argv)
     yaz_enable_panic_backtrace(*argv);
     read_params(argc, argv, &config);
 
-    if (config.conffile && strlen(config.conffile))
+    if (strlen(config.conffile))
         process_text_file(&config);
 
-    if (config.print && strlen(config.print))
+    if (strlen(config.print))
         print_info(&config);
 
     u_cleanup();
@@ -585,4 +595,3 @@ int main(int argc, char **argv)
  * End:
  * vim: shiftwidth=4 tabstop=8 expandtab
  */
-
